﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive.Views;
using HeuristicLab.MainForm;

namespace HeuristicLab.Clients.Hive.JobManager.Views {
  public partial class HiveResourceSelectorDialog : Form {
    private readonly object locker = new object();
    private bool updatingProjects = false;

    public Project SelectedProject {
      get { return hiveResourceSelector.SelectedProject; }
    }

    public IEnumerable<Resource> SelectedResources {
      get { return hiveResourceSelector.SelectedResources; }
    }

    public HiveResourceSelectorDialog() {
      InitializeComponent();
    }

    #region Overrides
    protected override void OnLoad(EventArgs e) {
      HiveAdminClient.Instance.Refreshing += HiveAdminClient_Instance_Refreshing;
      HiveAdminClient.Instance.Refreshed += HiveAdminClient_Instance_Refreshed;
      base.OnLoad(e);
    }

    protected override void OnClosing(CancelEventArgs e) {
      HiveAdminClient.Instance.Refreshed -= HiveAdminClient_Instance_Refreshed;
      HiveAdminClient.Instance.Refreshing -= HiveAdminClient_Instance_Refreshing;
      base.OnClosing(e);
    }
    #endregion

    #region Event Handlers
    private void HiveAdminClient_Instance_Refreshing(object sender, EventArgs e) {
      if (InvokeRequired) Invoke((Action<object, EventArgs>)HiveAdminClient_Instance_Refreshing, sender, e);
      else {
        var mainForm = MainFormManager.GetMainForm<MainForm.WindowsForms.MainForm>();
        mainForm.AddOperationProgressToView(this, "Refreshing ...");
        refreshButton.Enabled = false;
      }
    }

    private void HiveAdminClient_Instance_Refreshed(object sender, EventArgs e) {
      if (InvokeRequired) Invoke((Action<object, EventArgs>)HiveAdminClient_Instance_Refreshed, sender, e);
      else {
        var mainForm = MainFormManager.GetMainForm<MainForm.WindowsForms.MainForm>();
        mainForm.RemoveOperationProgressFromView(this);
        refreshButton.Enabled = true;
      }
    }

    private async void HiveResourceSelectorDialog_Load(object sender, EventArgs e) {
      lock (locker) {
        if (updatingProjects) return;
        updatingProjects = true;
      }

      await SecurityExceptionUtil.TryAsyncAndReportSecurityExceptions(
        action: () => UpdateProjects(),
        finallyCallback: () => updatingProjects = false);
    }

    private async void refreshButton_Click(object sender, EventArgs e) {
      lock (locker) {
        if (updatingProjects) return;
        updatingProjects = true;
      }

      await SecurityExceptionUtil.TryAsyncAndReportSecurityExceptions(
        action: () => UpdateProjects(),
        finallyCallback: () => updatingProjects = false);
    }

    private void hiveResourceSelector_SelectedProjectChanged(object sender, EventArgs e) {
      okButton.Enabled = hiveResourceSelector.SelectedProject != null;
    }

    private void hiveResourceSelector_SelectedResourcesChanged(object sender, EventArgs e) {
      okButton.Enabled = hiveResourceSelector.SelectedResources.Any();
    }

    private void hiveResourceSelector_ProjectsTreeViewDoubleClicked(object sender, EventArgs e) {
      if (hiveResourceSelector.SelectedProject == null) return;

      DialogResult = DialogResult.OK;
      Close();
    }
    #endregion

    #region Helpers
    private void UpdateProjects() {
      HiveAdminClient.Instance.Refresh();
      hiveResourceSelector.Content = HiveAdminClient.Instance.Projects;
    }

    private void ShowHiveInformationDialog() {
      if (InvokeRequired) Invoke((Action)ShowHiveInformationDialog);
      else {
        using (HiveInformationDialog dialog = new HiveInformationDialog()) {
          dialog.ShowDialog(this);
        }
      }
    }
    #endregion
  }
}
